<?php
namespace Zeedhi\Tools\Generator;

use Zeedhi\Model\Util\Filters\Connection;
use Zeedhi\Tools\DatabaseReader\DatabaseReader;
use Zeedhi\Tools\EntityGenerator\ClassGenerator;
use Zeedhi\Tools\EntityGenerator\MappingGenerator;
use Zeedhi\Tools\EntityGenerator\MetaData;
use Zeedhi\Tools\TableImporter\TableImporter;


class FacadeGenerator {

    /** @var string[] */
    protected $tablesToGenerate = array();
    /** @var mixed[] */
    protected $data = array();
    /** @var mixed[] */
    protected $metaData = array();
    /** @var GeneratorConfig */
    protected $config;
    /** @var TableImporter */
    protected $tableImporter;
    /** @var MetaData */
    protected $entityMetadataGenerator;
    /** @var MappingGenerator */
    protected $entityMappingFilesGenerator;
    /** @var ClassGenerator */
    protected $entityClassFilesGenerator;
    /** @var string */
    protected $entitiesNames = array();


    /**
     * @param GeneratorConfig $config
     */
    public function __construct(GeneratorConfig $config) {
        $this->config = $config;
    }

    protected function initTableImporter($clean){
        $connection = $this->config->getconnectionConfig()->factoryConnection();
        $dataBaseReader = DatabaseReader::factoryFromConnection($connection, $this->config->getDbReaderExtraConfig());
        $this->tableImporter = new TableImporter($dataBaseReader, $clean);
    }

    protected function initMetaDataGenerator($clean, $verbose){
        $inflector = $this->config->getInflector();
        $this->entityMetadataGenerator = new MetaData($inflector, $clean, $verbose, $this->config->getInverseRelations());
    }

    protected function initMappingGenerator($verbose){
        $this->entityMappingFilesGenerator = new MappingGenerator($verbose);
        if ($this->config->getMappingType()) {
            $this->entityMappingFilesGenerator->setDriver($this->config->getMappingType());
        }
    }

    protected function initClassFilesGenerator($verbose){
        $this->entityClassFilesGenerator = new ClassGenerator($verbose);
        $this->entityClassFilesGenerator->setPsr4Root($this->config->getPsr4Root());
    }

    protected function init($clean, $verbose){
        $this->initTableImporter($clean);
        $this->initMetaDataGenerator($clean, $verbose);
        $this->initMappingGenerator($verbose);
        $this->initClassFilesGenerator($verbose);
        $this->initialized = true;
    }

    protected function generateTablesData($clean){
        $this->importTables($clean);
        $this->data = $this->tableImporter->getData();
    }

    protected function generateMetaData(){
        $this->importEntities();
        $this->metaData = $this->entityMetadataGenerator->getMetaData();
    }

    protected function generateClasses(){
        $this->createFiles();
    }

    protected function importTables($clean){
        $tables = $this->getSubModelsByTableFromConfig();
        $this->tablesToUse = [];
        if($clean && $this->tablesToGenerate){
            foreach ($this->tablesToGenerate as $table) {
                if(isset($tables[$table])){
                    $this->tablesToUse[$table] = $tables[$table];
                }
            }
        }
        $this->tablesToUse = $this->tablesToUse ?: $tables;
        $this->tableImporter->importTablesWithSubModel($this->tablesToUse);
    }

    protected function importEntities(){
        $this->entityMetadataGenerator->generateEntitiesForTableNames($this->data, $this->config->getNamespace(), $this->tablesToUse);
    }

    protected function createFiles(){
        $entitiesToUse = [];
        foreach ($this->tablesToGenerate as $table) {
            if(isset($this->metaData["ENTITY"][$table])){
                $entitiesToUse[$table] = $this->metaData["ENTITY"][$table];
            }
        }

        $entitiesToUse = count($entitiesToUse) ? $entitiesToUse : $this->metaData["ENTITY"];

        $path = $this->config->getMappingPath();

        $this->entityMappingFilesGenerator->generateMappingForEntities(
            $entitiesToUse,
            $path,
            $this->metaData,
            $this->data
        );

        $this->entityClassFilesGenerator->createClassesFiles($this->config, $this->metaData, $entitiesToUse);
    }

    public function generate($clean = false, $verbose = false){
        try {
            $this->init($clean, $verbose);
            $this->generateTablesData($clean);
            $this->generateMetaData();
            $this->generateClasses();
        } catch(\Exception $e) {
            echo "[critical error] ", $e->getMessage(),  PHP_EOL, $e->getTraceAsString();
        }
    }

    public static function factory($pathToConfigurationFile){
        return new static(GeneratorConfig::factoryFromJSONString(file_get_contents($pathToConfigurationFile)));
    }

    /**
     * @return array
     */
    protected function getSubModelsByTableFromConfig(){
        return $this->config->getSubModelsByTable();
    }

    /**
     * @return array
     * @throws \Exception
     */
    protected function getTablesWithSubModels(){
        $subModelsByTableFromConfig = $this->getSubModelsByTableFromConfig();
        $subModelsByTable = array();
        foreach($this->tablesToGenerate as $tableName) {
            if (!isset($subModelsByTableFromConfig[$tableName])) {
                throw new \Exception("Table {$tableName} not found in entities.json.");
            }

            $subModelsByTable[$tableName] = $subModelsByTableFromConfig[$tableName];
        }

        return count($subModelsByTable) ? $subModelsByTable : $subModelsByTableFromConfig;
    }

    /**
     * @param \string[] $tablesToGenerate
     */
    public function setTablesToGenerate($tablesToGenerate){
        $this->tablesToGenerate = $tablesToGenerate;
    }
}